<li>
    <div class='element-wrapper'>
        <span class='path'>{{path}}</span>
        <span class='newElement'>
            <span class="menu element" style='cursor:pointer;'> {{namespace}}:element :</span>
        </span>
        <span class='name'>{{type_name}}</span>
        <span class='type'>{{type_name}}</span>
        <span class='occurs'>( 1 , 1)</span>
    </div>
</li>