.options_menu{
    position:absolute;
    z-index:9999;
    border-radius: 3px;
    border: 1px solid #cacaca;
}

.menu{
    cursor:pointer;
}

.options_menu .dropdown-item {
    font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, "Noto Sans", sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
    margin: 0;
    cursor: pointer;
    font-size: 0.8em;
    padding: 0 10px 0 10px;
}

.modal-body td {
    vertical-align: middle;
    padding-left: 5px
}