{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}

{% block modal_id %}delete-element-modal{% endblock %}
{% block modal_title %}Delete Element{% endblock %}

{% block modal_body %}
Are you sure you want to delete this element from the current template?
{% endblock %}
{% block modal_footer %}
<button class="btn btn-secondary pull-left" data-dismiss="modal"><i class="fa fa-times"></i> Cancel</button>
<button id="delete-element" class="btn btn-danger"><i class="fa fa-trash"></i> Delete</button>
{% endblock %}
