<div id="menu_element" class="options_menu">
    <a class="menu dropdown-item rename-element" href="#">Rename</a>
    <a class="menu dropdown-item element-occurrences" href="#">Manage Occurrences</a>
    <a class="menu dropdown-item delete-element" href="#">Delete</a>
</div>
