{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}

{% block modal_id %}save-success-modal{% endblock %}
{% block modal_title %}XML Schema Saved{% endblock %}

{% block modal_body %}
Saved to repository successfully.
{% endblock %}
{% block modal_footer %}
<div class="w-100 d-flex flex-row-reverse">
    <a class="btn btn-primary" data-dismiss="modal" href="#">
        <i class="fa fa-check"></i> OK
    </a>
</div>
{% endblock %}

