{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}

{% block modal_id %}error-modal{% endblock %}
{% block modal_title %}Error{% endblock %}

{% block modal_body %}
An error occurred:
<div id="validate-error" class="text-danger">
</div>
{% endblock %}
{% block modal_footer %}
<a class="btn btn-secondary pull-left" data-dismiss="modal" href="#"><i class="fa fa-times"></i> Cancel</a>
{% endblock %}

