{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}

{% block modal_id %}occurrences-modal{% endblock %}
{% block modal_title %}Manage Occurrences{% endblock %}

{% block modal_body %}
Use the following form to manage the occurrences of the selected element.
Accepted values are integers.
<table>
    <tr>
        <td>Minimum Occurrences:</td>
        <td>
            <input type="text" id="minOccurrences"/>
        </td>
    </tr>
    <tr>
        <td>Maximum Occurrences:</td>
        <td>
            <input type="text" id="maxOccurrences"/>
        </td>
        <td>
            <label>Unbounded</label>
        </td>
        <td>
            <input type="checkbox" id="unbounded"/>
        </td>
    </tr>
</table>
<div id="manage-occurrences-error" class="text-danger">
</div>
{% endblock %}
{% block modal_footer %}
<a class="btn btn-secondary pull-left" data-dismiss="modal" href="#"><i class="fa fa-times"></i> Cancel</a>
<a id="set-occurrences" class="btn btn-primary" href="#">
    <i class="fa fa-sync"></i> Update Occurrences
</a>
{% endblock %}
